"""Specialized agent demo modules."""

from examples.demos.security_auditor_demo import demonstrate_security_auditor
from examples.demos.code_reviewer_demo import demonstrate_code_reviewer
from examples.demos.solution_architect_demo import demonstrate_solution_architect
from examples.demos.agent_collaboration_demo import demonstrate_agent_collaboration

__all__ = [
    "demonstrate_security_auditor",
    "demonstrate_code_reviewer",
    "demonstrate_solution_architect",
    "demonstrate_agent_collaboration",
]
