"""Cross-Project Learning - Share knowledge across projects"""
from .cross_project import (
    CrossProjectLearning,
    ProjectProfile,
    KnowledgeEntry,
    InsightType,
    get_cross_project_learning,
)

__all__ = [
    "CrossProjectLearning",
    "ProjectProfile",
    "KnowledgeEntry",
    "InsightType",
    "get_cross_project_learning",
]
